@@ -1,5 +1,5 @@
 # sql/ddl.py
-# Copyright (C) 2009-2021 the SQLAlchemy authors and contributors
+# Copyright (C) 2009-2022 the SQLAlchemy authors and contributors
 # <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
@@ -312,8 +312,10 @@ class DDL(DDLElement):
 
         :param statement:
           A string or unicode string to be executed.  Statements will be
-          processed with Python's string formatting operator.  See the
-          ``context`` argument and the ``execute_at`` method.
+          processed with Python's string formatting operator using
+          a fixed set of string substitutions, as well as additional
+          substitutions provided by the optional :paramref:`.DDL.context`
+          parameter.
 
           A literal '%' in a statement must be escaped as '%%'.
 
